--[[
The lmb_ blocks are from LMB and LMBB:
Custom Mods For Creative Server by LandMine, Version 9.15.12
--]]

-- ===================================================================

local modname   = minetest.get_current_modname()
local modpath   = minetest.get_modpath (modname)
local modprefix = modname .. ":"

local reg_alias = ocutil.safe_register_alias
local reg_item  = ocutil.safe_register_item
local reg_node  = ocutil.safe_register_node
local reg_tool  = ocutil.safe_register_tool

local needitem

-- ===================================================================

coderblocks = {}

-- ===================================================================

coderblocks.is_node = function (name)
    if minetest.registered_nodes [name] == nil then
        return false
    else
        return true
    end
end

-- ===================================================================

coderblocks.not_node = function (name)
    if minetest.registered_nodes [name] == nil then
        return true
    else
        return false
    end
end

-- ===================================================================

local function register_lmbnode (name, desc)
    local lmb_name  = "lmb_" .. name
    local lmb_image = lmb_name .. ".png"

    if name == "old_map" then
       lmb_image = lmb_name .. ".jpg"
    end

    reg_node (modprefix .. lmb_name    , {
        description       = desc                   ,
        tiles             = { lmb_image }          ,
        is_ground_content = true                   ,
        groups            = { cracky = 3         } ,
        sounds = default.sound_stone() ,

	light_source      = 6           ,
	paramtype         = "light"	,
    })

    reg_alias ("lmb:"      .. name     ,
               modprefix   .. lmb_name )
    reg_alias ("ocblocks:" .. name     ,
               modprefix   .. lmb_name )
end

-- ===================================================================

register_lmbnode ("apple_crate"    , "LMB apple crate"    )
register_lmbnode ("asphalt"        , "LMB asphalt"        )
register_lmbnode ("checkered_wood" , "LMB checkered wood" )
register_lmbnode ("checkered_wool" , "LMB checkered wool" )
register_lmbnode ("cheese"         , "LMB cheese"         )
register_lmbnode ("chessboard"     , "LMB chessboard"     )
register_lmbnode ("chocolate"      , "LMB chocolate"      )
register_lmbnode ("clock"          , "LMB clock"          )
register_lmbnode ("coal"           , "LMB coal block"     )
register_lmbnode ("cobble"         , "LMB cobble block"   )
register_lmbnode ("concrete"       , "LMB concrete"       )
register_lmbnode ("crate"          , "LMB crate"          )
register_lmbnode ("creeper"        , "LMB creeper"        )
register_lmbnode ("diamond"        , "LMB diamond"        )
register_lmbnode ("green_rock"     , "LMB green rock"     )
register_lmbnode ("gunpowder"      , "LMB gunpowder"      )
register_lmbnode ("hardrock"       , "LMB hard rock"      )
register_lmbnode ("haybale"        , "LMB haybale"        )
register_lmbnode ("large_brick"    , "LMB large brick"    )
register_lmbnode ("lava_rock"      , "LMB lava rock"      )
register_lmbnode ("limestone"      , "LMB limestone"      )
register_lmbnode ("marble"         , "LMB marble"         )
register_lmbnode ("marble_tile"    , "LMB marble tile"    )
register_lmbnode ("pink_wood"      , "LMB pink wood"      )
register_lmbnode ("potion_shelf"   , "LMB potion shelf"   )
register_lmbnode ("redstone"       , "LMB redstone"       )
register_lmbnode ("sand_bricks"    , "LMB sand bricks"    )
register_lmbnode ("slime"          , "LMB slime block"    )
register_lmbnode ("square"         , "LMB square block"   )
register_lmbnode ("stone"          , "LMB stone block"    )
register_lmbnode ("yellow_wood"    , "LMB yellow wood"    )

-- ===================================================================

-- Special case (not one of the original nodes)
--
register_lmbnode ("old_map", "LMB old map")

reg_alias ("lmb:checkeredwood"                ,
           modprefix .. "lmb_checkered_wood"  )

reg_alias ("lmb:checkeredwool"                ,
           modprefix .. "lmb_checkered_wool"  )

reg_alias ("lmb:coalblock"                    ,
           modprefix .. "lmb_coal"            )

reg_alias ("lmb:cobbleblock"                  ,
           modprefix .. "lmb_cobble"          )

reg_alias ("lmb:gunpowderblock"               ,
           modprefix .. "lmb_gunpowder"       )

reg_alias ("lmb:largebrick"                   ,
           modprefix .. "lmb_large_brick"     )

reg_alias ("lmb:lavarock"                     ,
           modprefix .. "lmb_lava_rock"       )

reg_alias ("lmb:map"                          ,
           modprefix .. "lmb_old_map"         )

reg_alias ("lmb:marbletile"                   ,
           modprefix .. "lmb_marble_tile"     )

reg_alias ("lmb:sandbricks"                   ,
           modprefix .. "lmb_sand_bricks"     )

reg_alias ("lmb:slimeblock"                   ,
           modprefix .. "lmb_slime"           )

reg_alias ("lmb:squareblock"                  ,
           modprefix .. "lmb_square"          )

-- ===================================================================

reg_alias ("lmbb:applecrate"                  ,
           modprefix .. "lmb_apple_crate"     )

reg_alias ("lmbb:creeperblock"                ,
           modprefix .. "lmb_creeper"         )

reg_alias ("lmbb:hardrock"                    ,
           modprefix .. "lmb_hardrock"        )

reg_alias ("lmbb:potionshelf"                 ,
           modprefix .. "lmb_potion_shelf"    )

reg_alias ("lmbb:redstoneblock"               ,
           modprefix .. "lmb_redstone"        )

reg_alias ("lmbb:yellowwood"                  ,
           modprefix .. "lmb_yellow_wood"     )

-- ===================================================================

local lmb_image_png

-- ===================================================================

lmb_image_png = "lmb_wall_map.png"

reg_node (modprefix .. "lmb_wall_map", {
    description = "LMB wall map",
    drawtype = "signlike",
    tiles = { lmb_image_png },
    inventory_image = lmb_image_png,
    wield_image = lmb_image_png,
    paramtype = "light",
    paramtype2 = "wallmounted",
    is_ground_content = true,
    walkable = false,
    climbable = false,
    selection_box = {
        type = "wallmounted",
    },

    groups = {snappy=3,flammable=2},
    sounds = default.sound_leaves(),
    legacy_wallmounted = true,
})

reg_alias ("ocblocks:lmb_wall_map"     ,
           modprefix .. "lmb_wall_map" )

-- ===================================================================

lmb_image_png = "lmb_goldblock.png"

reg_node (modprefix .. "lmb_goldblock", {
    description       = "LMB Gold Block"  ,
    tiles             = { lmb_image_png } ,
    paramtype         = "light"           ,
    light_source      = 7                 ,
    is_ground_content = true              ,
    groups            = { cracky=3 }      ,
    sounds = default.sound_stone() ,
})

reg_alias ("lmb:goldblock"                ,
           modprefix .. "lmb_goldblock"   )
reg_alias ("ocblocks:lmb_goldblock"       ,
           modprefix .. "lmb_goldblock"   )

-- ===================================================================

lmb_image_png = "lmb_glowstone.png"

reg_node (modprefix .. "lmb_glowstone", {
    description         = "LMB glowstone"       ,
    drawtype            = "glasslike"           ,
    tiles               = { lmb_image_png }     ,
    inventory_image     =   lmb_image_png       ,
    paramtype           = "light"               ,
    sunlight_propagates = true                  ,
    light_source        = default.LIGHT_MAX     ,
    is_ground_content   = true                  ,

    groups = { snappy=2, cracky=3, oddly_breakable_by_hand=3 }   ,
    sounds = default.sound_glass()              ,
})

reg_alias ("lmbb:glowstone"             ,
           modprefix .. "lmb_glowstone" )
reg_alias ("ocblocks:lmb_glowstone"     ,
           modprefix .. "lmb_glowstone" )

-- ===================================================================

-- Rescued from "special_picks".

reg_node (modprefix .. "glass_steel_block", {
    description = "Glass Steel Block",
    drawtype = "glasslike",
    tiles = {"oldcoder_glass_steel_block.png"},
    paramtype = "light",
    sunlight_propagates = true,
    is_ground_content = true,
    groups = {cracky=1,level=2},
    sounds = default.sound_glass(),
})

reg_alias ("ocblocks:glass_steel_block"      ,
           modprefix .. "glass_steel_block"  )

reg_alias ("oldcoder:glass_steel_block"      ,
           modprefix .. "glass_steel_block"  )

-- ===================================================================

reg_node (modprefix .. "glowstone", {
    description = "Glowstone",
    tiles = {"glowstone_glowstone.png"},
    is_ground_content = true,
    light_source = 12,
    groups = { cracky=3, oddly_breakable_by_hand=3 } ,
    sounds = default.sound_glass() ,
})

minetest.register_craft ({
    output = modprefix .. "glowstone" ,
    recipe = {
        { '', 'group:stone', '' } ,
        { 'group:stone', 'default:torch', 'group:stone' } ,
        { '', 'group:stone', '' } ,
    }
})

reg_alias  ("ocblocks:glowstone"     ,
            modprefix .. "glowstone" )

reg_alias  ("oldcoder:glowstone"     ,
            modprefix .. "glowstone" )

reg_alias ("glowstone:block"         ,
           modprefix .. "glowstone"  )

reg_alias ("glowstone:glowstone"     ,
           modprefix .. "glowstone"  )

-- ===================================================================

reg_node (modprefix .. "permaleaves", {
    description       = "Permanent Leaves"          ,
    drawtype          = "allfaces_optional"         ,
    visual_scale      = 1.3                         ,
    tiles             = { "default_leaves.png" }    ,
    paramtype         = "light"                     ,
    waving            = 1                           ,
    is_ground_content = false                       ,

    groups = { snappy=3, flammable=2, leaves=1 }    ,
    sounds = default.sound_leaves() ,
})

reg_alias ("permaleaves:leaves"       ,
           modprefix .. "permaleaves" )

reg_alias ("permaleaves:permaleaves"  ,
           modprefix .. "permaleaves" )

-- ===================================================================
-- Useful for very large Christmas trees.

reg_node (modprefix .. "ornament_red" , {

    description       = "Ornament Red"                ,
    is_ground_content = false                         ,
    light_source      = 10                            ,
    sounds            = default.node_sound_defaults() ,
    tiles             = {"wool_red.png" }             ,

    groups = {
        snappy=2, choppy=2, oddly_breakable_by_hand=3,
flammable=3, wool=1
    } ,
})

reg_alias ("ocblocks:ornament_red"     ,
           modprefix .. "ornament_red" )

reg_alias ("oldcoder:ornament_red"     ,
           modprefix .. "ornament_red" )

-- ===================================================================
-- Rescued from an old diamonds mod.

minetest.register_tool (modprefix .. "pick", {
    description = "Oldcoder Pickaxe",
    inventory_image = "oldcoder_pick.png",
    tool_capabilities = {
        full_punch_interval = 1.0,
        max_drop_level=3,
        groupcaps={
            cracky  = {times={[1]=2.0, [2]=1.0, [3]=0.5}, uses=500, maxlevel=3},
            crumbly = {times={[1]=2.0, [2]=1.0, [3]=0.5}, uses=600, maxlevel=3},
            snappy  = {times={[1]=2.0, [2]=1.0, [3]=0.5}, uses=550, maxlevel=3}
        }
    },
})

reg_alias ("ocblocks:pick"     ,
           modprefix .. "pick" )

reg_alias ("oldcoder:pick"     ,
           modprefix .. "pick" )

-- ===================================================================
-- Rescued from an old "industrial" mod.

local WBPNG = "oldcoder_white_brick.png"

reg_node (modprefix .. "white_brick", {

    description       = "White Brick"   ,
    tiles             = { WBPNG }       ,
    inventory_image   =   WBPNG         ,
    is_ground_content = true            ,
    groups            = { cracky=3 }    ,
    sounds            = default.sound_stone() ,
})

reg_alias ("ocblocks:white_brick"     ,
           modprefix .. "white_brick" )

reg_alias ("oldcoder:white_brick"     ,
           modprefix .. "white_brick" )

-- ===================================================================
-- These coins are adapted from the "maptools" mod.

reg_item (modprefix .. "copper_coin", {
    description     = "Copper Coin"                  ,
    inventory_image = "coderblocks_copper_coin.png"  ,
    wield_scale     = { x = 0.5, y = 0.5, z = 0.25 } ,
    stack_max       = 10000                          ,
})

reg_item (modprefix .. "silver_coin", {
    description     = "Silver Coin"                  ,
    inventory_image = "coderblocks_silver_coin.png"  ,
    wield_scale     = { x = 0.5, y = 0.5, z = 0.25 } ,
    stack_max       = 10000                          ,
})

reg_item (modprefix .. "gold_coin", {
    description     = "Gold Coin"                    ,
    inventory_image = "coderblocks_gold_coin.png"    ,
    wield_scale     = { x = 0.5, y = 0.5, z = 0.25 } ,
    stack_max       = 10000                          ,
})

-- ===================================================================
-- From Shadow Server.

-- This node is useful  mostly because it illustrates how a modder can
-- sculpt a single node.

reg_node (modprefix .. "shadow_kitten", {
    description = "Kitten (AKA NekoGloop)"      ,
    tiles       = { "wool_black.png" }          ,
    groups      = { oddly_breakable_by_hand=1 } ,
    drawtype    = "nodebox"                     ,
    paramtype   = "light"                       ,

    node_box    = {
        type  = "fixed" ,
        fixed = {
{ -0.2500, -0.3125, -0.2500,  0.2500,  0.1250,  0.5000 } , -- Body
{ -0.1875, -0.1250, -0.5000,  0.1875,  0.2500, -0.2500 } , -- Head
{  0.1250, -0.5000,  0.2500,  0.2500, -0.3125,  0.4500 } , -- Leg
{  0.1250, -0.5000, -0.1250,  0.2500, -0.3125,  0.0625 } , -- Leg
{ -0.2500, -0.5000,  0.2500, -0.1250, -0.3125,  0.4625 } , -- Leg
{ -0.2500, -0.5000, -0.1250, -0.1250, -0.3125,  0.0625 } , -- Leg
        }
    }
})

-- ===================================================================

mname = "mobs:cobweb"
--
if minetest.registered_nodes [mname] == nil then
reg_node (":" .. mname, {
    description = "Cobweb",
    drawtype = "plantlike",
    visual_scale = 1.2,
    tiles = {"coderblocks_cobweb.png"} ,
    inventory_image = "coderblocks_cobweb.png",
    paramtype = "light",
    sunlight_propagates = true,
    liquid_viscosity = 11,
    liquidtype = "source",
    liquid_alternative_flowing = "mobs:cobweb",
    liquid_alternative_source = "mobs:cobweb",
    liquid_renewable = false,
    liquid_range = 0,
    walkable = false,
    groups = {snappy = 1, disable_jump = 1} ,
    drop = "farming:cotton",
    sounds = default.sound_leaves(),
})

minetest.register_craft({
    output = "mobs:cobweb",
    recipe = {
        {"farming:string", "", "farming:string"} ,
        {"", "farming:string", ""} ,
        {"farming:string", "", "farming:string"} ,
    }
})
end

-- ===================================================================

if bucket.register_bucket_hash then
   bucket.register_bucket_hash ({
       source_node     = "coderblocks:lmb_cheese"  ,
       bucket_node     = "bucket:cheese"           ,
       desc            = "Bucket O'Cheese"         ,
       invsource_image = "lmb_cheese.png"          ,
    })
end

-- ===================================================================

reg_node ("coderblocks:ornament_dark_green" , {
    description = "Ornament Dark Green" ,
    tiles = {"wool_dark_green.png" } ,
    is_ground_content = false,
    light_source = 9,
    groups = {snappy = 2, choppy = 2, oddly_breakable_by_hand = 3,
    flammable = 3, wool = 1 } ,
    sounds = default.node_sound_defaults(),
})

reg_node ("coderblocks:wool_glow_magenta" , {
    description = "Magenta Glow-Wool" ,
    tiles = {"wool_magenta.png" } ,
    is_ground_content = false,
    light_source = 9,
    groups = {snappy = 2, choppy = 2, oddly_breakable_by_hand = 3,
    flammable = 3, wool = 1 } ,
    sounds = default.node_sound_defaults(),
})

reg_node ("coderblocks:wool_glow_white" , {
    description = "White Glow-Wool" ,
    tiles = {"wool_white.png" } ,
    is_ground_content = false,
    light_source = 9,
    groups = {snappy = 2, choppy = 2, oddly_breakable_by_hand = 3,
    flammable = 3, wool = 1 } ,
    sounds = default.node_sound_defaults(),
})

reg_node ("coderblocks:wool_glow_yellow" , {
    description = "Yellow Glow-Wool" ,
    tiles = {"wool_yellow.png" } ,
    is_ground_content = false,
    light_source = 9,
    groups = {snappy = 2, choppy = 2, oddly_breakable_by_hand = 3,
    flammable = 3, wool = 1 } ,
    sounds = default.node_sound_defaults(),
})

reg_node ("coderblocks:wool_glow_black" , {
    description = "Black Glow-Wool" ,
    tiles = {"wool_black.png" } ,
    is_ground_content = false,
    light_source = 11,
    groups = {snappy = 2, choppy = 2, oddly_breakable_by_hand = 3,
    flammable = 3, wool = 1 } ,
    sounds = default.node_sound_defaults(),
})

-- ===================================================================

reg_node ("coderblocks:astrodirt" , {
    description = "Dirt" ,
    tiles  = { "default_dirt.png" } ,
    groups = { crumbly = 3, soil = 1 } ,
    sounds = default.sound_dirt(),
})

reg_node ("coderblocks:astroturf", {
    description = "Astroturf Block",
    tiles  = { "default_grass.png" } ,
    is_ground_content = true,
    groups = {crumbly=3} ,
    sounds = default.sound_dirt({
        footstep = {name="default_grass_footstep", gain=0.4} ,
    }),
})

reg_alias ("farming_plus:docgrass"    ,
           "coderblocks:astroturf"    )

-- ===================================================================

reg_node ("coderblocks:coderturf", {
    description = "Coderturf Block",
    tiles  = { "default_grass.png" } ,
    is_ground_content = true,
    groups = {crumbly=3} ,
    sounds = default.sound_dirt({
        footstep = {name="default_grass_footstep", gain=0.4} ,
    }),
})

-- ===================================================================

meowza = {}

-- Node will be called <modname>micro_<subname>

coderblocks.micro = function (modname, subname, recipeitem, groups, images, description, drop)
    reg_node (modname .. ":micro_" .. subname .. "_bottom", {
        description = description,
        drawtype = "nodebox",
        tiles = images,
        drop = modname .. ":micro_" .. drop .. "_bottom",
        paramtype = "light",
        paramtype2 = "facedir",
        sunlight_propagates = true,
        is_ground_content = true,
        groups = groups,
        node_box = {
            type = "fixed",
            fixed = {-0.5, -0.5, 0, 0, 0, 0.5} ,
        } ,
        selection_box = {
            type = "fixed",
            fixed = {-0.5, -0.5, 0, 0, 0, 0.5} ,
        } ,
        sounds = default.sound_stone(),
        on_place = function(itemstack, placer, pointed_thing)
    if pointed_thing.type ~= "node" then
        return itemstack
    end

    local p0 = pointed_thing.under
    local p1 = pointed_thing.above
    if p0.y-1 == p1.y then
        local fakestack = ItemStack(modname .. ":micro_" .. subname.. "_top")
        local ret = minetest.item_place(fakestack, placer, pointed_thing)
        if ret:is_empty() then
            itemstack:take_item()
            return itemstack
        end
    end

    -- Otherwise place regularly
    return minetest.item_place(itemstack, placer, pointed_thing)
    end,
    })

    reg_node (modname .. ":micro_" .. subname .. "_top", {
        description = description,
        drawtype = "nodebox",
        tiles = images,
        drop = modname .. ":micro_" .. drop .. "_top",
        paramtype = "light",
        paramtype2 = "facedir",
        sunlight_propagates = true,
        is_ground_content = true,
        groups = groups,
        node_box = {
            type = "fixed",
            fixed = {-0.5, 0, 0, 0, 0.5, 0.5} ,
        } ,
        selection_box = {
            type = "fixed",
            fixed = {-0.5, 0, 0, 0, 0.5, 0.5} ,
        } ,
        sounds = default.sound_stone(),
    })

    minetest.register_craft({
        output = modname .. ":micro_" .. subname .. "_bottom 8",
        recipe = {
            {"default:stick"} ,
            {recipeitem} ,
        } ,
    })

    minetest.register_craft({
        output = modname .. ":micro_" .. subname .. "_top 1",
        recipe = {
            {modname .. ":micro_" .. subname .. "_bottom"} ,
        } ,
    })

    minetest.register_craft({
        output = modname .. ":micro_" .. subname .. "_bottom 1",
        recipe = {
            {modname .. ":micro_" .. subname .. "_top"} ,
        } ,
    })
end

-- Nodes will be called <modname>:{stair,slab,panel,micro}_<subname>
--
function meowza.register_stair_and_slab_and_panel_and_micro(modname, subname, recipeitem, groups, images, desc_stair, desc_slab, desc_panel, desc_micro, drop)
--  meowza.register_stair(modname, subname, recipeitem, groups, images, desc_stair, drop)
--  meowza.register_slab(modname, subname, recipeitem, groups, images, desc_slab, drop)
--  meowza.register_panel(modname, subname, recipeitem, groups, images, desc_panel, drop)
    meowza.register_micro(modname, subname, recipeitem, groups, images, desc_micro, drop)
end

if coderblocks.not_node ("poudlard:micro_steelblock_bottom") then
coderblocks.micro (":poudlard",
    "steelblock", "default:steelblock" ,
        {snappy=1,bendy=2,cracky=1,melty=2,level=2} ,
        {"default_steel_block.png"} ,
        "Steel Block Stairs",
        "Steel Block Slab",
        "Steel Block Panel",
        "Steel Microblock",
        "steelblock",
        "none")
end

-- ===================================================================

reg_node ("coderblocks:fake_fire", {
    description = "Fake Fire" ,
    range = 12,
    stack_max = 10000,
    drawtype = "plantlike",
    paramtype = "light",
    tiles = {{
        name="coderblocks_fake_fire_animated.png" ,
        animation = {
            type="vertical_frames", aspect_w=16, aspect_h=16, length=1
        } ,
    }} ,
    inventory_image = "coderblocks_fake_fire_inv.png",
    light_source = 14,
    drop = "",
    groups = {
        dig_immediate=3
    } ,
    sunlight_propagates = true,
    walkable = false,
})

-- ===================================================================

reg_node ("coderblocks:hdspeaker", {
    description = "Large Stereo Speaker",
    mesh="coderblocks_speaker_large.obj",
    tiles = {
        "coderblocks_speaker_sides.png" ,
        "coderblocks_speaker_front.png" ,
    } ,
    groups = { snappy = 3 } ,
    sounds = default.sound_wood(),
})

-- ===================================================================

reg_node ("coderblocks:hdtv", {
    description = "Small CRT Television" ,
    tiles = { 'coderblocks_television_top.png',
          'coderblocks_television_bottom.png',
          'coderblocks_television_left.png^[transformFX',
          'coderblocks_television_left.png',
          'coderblocks_television_back.png',
           { name="coderblocks_television_front_animated.png",
              animation={
                type="vertical_frames",
                aspect_w=16,
                aspect_h=16,
                length=80.0
              }
           }
    } ,

    light_source = default.LIGHT_MAX - 1,
    groups = { snappy = 3 } ,
    sounds = default.sound_wood(),
})

-- ===================================================================

local function slab_z (depth)
    if depth > 0 then
        -- slab starting from -z
        return { -0.5, -0.5, -0.5, 0.5, 0.5, -0.5+depth }
    else
        -- slab starting from +z (z1=0.5-(-depth))
        return { -0.5, -0.5, 0.5+depth, 0.5, 0.5, 0.5 }
    end
end

-- ===================================================================

reg_node ("coderblocks:hdchainlink" , {
    description = "Chainlink Fence",
    mesh="coderblocks_fence_chainlink.obj",
    tiles = {
        "coderblocks_fence_chainlink_tb.png",
        "coderblocks_fence_chainlink_tb.png",
        "coderblocks_fence_chainlink_sides.png",
        "coderblocks_fence_chainlink_sides.png",
        "coderblocks_fence_chainlink_fb.png",
        "coderblocks_fence_chainlink_fb.png",
    } ,
    groups = {snappy=3} ,
    sounds = default.sound_wood(),
    selection_box = slab_z(-0.125),
    collision_box = slab_z(-0.125),
})

reg_node ("coderblocks:hdstereo", {
    description = "Stereo Receiver" ,
    tiles = { 'coderblocks_stereo_top.png',
            'coderblocks_stereo_bottom.png',
            'coderblocks_stereo_left.png^[transformFX',
            'coderblocks_stereo_left.png',
            'coderblocks_stereo_back.png',
            'coderblocks_stereo_front.png'} ,
    groups = { snappy = 3 } ,
    sounds = default.sound_wood(),
})

-- ===================================================================

reg_alias ("technic:marble_bricks"              ,
           "lottblocks:marble_brick"            )
reg_alias ("moretrees:pine_planks"              ,
           "default:wood"                       )

reg_alias ("protector:chest"                    ,
           "default:chest"                      )
reg_alias ("axinitium:dirt"                     ,
           "coderblocks:astrodirt"              )

reg_alias ("default:brick_technic_cnc_slope"    ,
           "stairs:slope_brick"                 )
reg_alias ("technic:gold_locked_chest_green"    ,
           "default:chest_locked"               )
reg_alias ("technic:mineral_zinc"               ,
           "default:stone"                      )
reg_alias ("glow:stone_with_worms"              ,
           "default:stone"                      )

-- ===================================================================

needitem = "shooter:rifle"
--
if ocutil.thing_missing (needitem) then
reg_item (needitem, {
    description     = "Rifle"             ,
    inventory_image = "shooter_rifle.png" ,
})
end

-- ===================================================================

needitem = "shooter:rocket_gun_loaded"
--
if ocutil.thing_missing (needitem) then
reg_item (needitem, {
    description     = "Rocket Gun Loaded"             ,
    inventory_image = "shooter_rocket_gun_loaded.png" ,
})
end

reg_node ("mcl_core:slimeblock", {
    description = "Slime Block" ,
    drawtype = "nodebox",
    paramtype = "light",
    is_ground_content = false,
    node_box = {
        type = "fixed",
        fixed = {
            {-0.25, -0.25, -0.25, 0.25, 0.25, 0.25}, 
            {-0.5, -0.5, -0.5, 0.5, 0.5, 0.5},
        }
    },
    selection_box = {
        type = "regular",
    } ,
    tiles = { "mcl_core_slime.png" } ,
    paramtype = "light",
    use_texture_alpha = true,
    stack_max = 64,

-- According to Minecraft Wiki, bouncing off a slime block from a height off 255 blocks should result in a bounce height of 50 blocks
-- bouncy=44 makes the player bounce up to 49.6. This value was chosen by experiment.
-- bouncy=80 was chosen because it is higher than 66 (bounciness of bed)

    groups = {
        dig_immediate=3, bouncy=80,fall_damage_add_percent=-100,deco_block=1
    },
})

-- ===================================================================

if ocutil.node_missing ("ocean:prismarine") then
    dofile (modpath .. "/mcl_ocean.lua")
end

dofile (modpath .. "/air.lua"           )
dofile (modpath .. "/aluminum.lua"      )
dofile (modpath .. "/cement.lua"        )
dofile (modpath .. "/darkage.lua"       )
dofile (modpath .. "/decoblocks.lua"    )

dofile (modpath .. "/diamonds.lua"      )
dofile (modpath .. "/dice2.lua"         )
dofile (modpath .. "/emeralds.lua"      )
dofile (modpath .. "/grassblock.lua"    )
dofile (modpath .. "/infinitytools.lua" )

dofile (modpath .. "/mcl_ocean.lua"     )
dofile (modpath .. "/mcl_sponges.lua"   )
dofile (modpath .. "/modernpack.lua"    )
dofile (modpath .. "/myretro.lua"       )
dofile (modpath .. "/prefab.lua"        )

dofile (modpath .. "/quartz.lua"        )
dofile (modpath .. "/glowcrystals.lua"  )
dofile (modpath .. "/morebeds.lua"      )
dofile (modpath .. "/projlights.lua"    )
dofile (modpath .. "/refrigerator.lua"  )

dofile (modpath .. "/skylands.lua"      )
dofile (modpath .. "/titanium.lua"      )
dofile (modpath .. "/bandage.lua"       )
dofile (modpath .. "/misc.lua"          )

-- This must come after "glowcrystals.lua"
dofile (modpath .. "/lottother.lua"     )

dofile (modpath .. "/rbsword.lua"       )

-- ===================================================================
-- End of file.
